@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2011-2017 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2011-2021 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -18,8 +18,8 @@
 # along with Rapid Photo Downloader.  If not,
 # see <http://www.gnu.org/licenses/>.
 
-__author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2011-2017, Damon Lynch"
+__author__ = "Damon Lynch"
+__copyright__ = "Copyright 2011-2021, Damon Lynch"
 
 import pickle
 import os
@@ -29,46 +29,58 @@ from datetime import datetime
 import shutil
 import logging
 import locale
+
 try:
     # Use the default locale as defined by the LANG variable
-    locale.setlocale(locale.LC_ALL, '')
+    locale.setlocale(locale.LC_ALL, "")
 except locale.Error:
     pass
 
 from typing import Optional, Tuple
 
 
-from raphodo.interprocess import (BackupFileData, BackupResults, BackupArguments,
-                          WorkerInPublishPullPipeline)
+from raphodo.interprocess import (
+    BackupFileData,
+    BackupResults,
+    BackupArguments,
+    WorkerInPublishPullPipeline,
+)
 from raphodo.copyfiles import FileCopy
-from raphodo.constants import (FileType, DownloadStatus, BackupStatus)
+from raphodo.constants import FileType, DownloadStatus, BackupStatus
 from raphodo.rpdfile import RPDFile
 from raphodo.cache import FdoCacheNormal, FdoCacheLarge
 
 from raphodo.copyfiles import copy_file_metadata
 from raphodo.problemnotification import (
-    BackingUpProblems, BackupSubfolderCreationProblem, make_href, BackupOverwrittenProblem,
-    BackupAlreadyExistsProblem, FileWriteProblem
+    BackingUpProblems,
+    BackupSubfolderCreationProblem,
+    make_href,
+    BackupOverwrittenProblem,
+    BackupAlreadyExistsProblem,
+    FileWriteProblem,
 )
-from raphodo.storage import get_uri
+from raphodo.storage.storage import get_uri
 
 
 class BackupFilesWorker(WorkerInPublishPullPipeline, FileCopy):
     def __init__(self):
         self.problems = BackingUpProblems()
-        super().__init__('BackupFiles')
+        super().__init__("BackupFiles")
 
     def update_progress(self, amount_downloaded, total):
         self.amount_downloaded = amount_downloaded
         chunk_downloaded = amount_downloaded - self.bytes_downloaded
         if (chunk_downloaded > self.batch_size_bytes) or (amount_downloaded == total):
             self.bytes_downloaded = amount_downloaded
-            self.content= pickle.dumps(BackupResults(
-                scan_id=self.scan_id,
-                device_id=self.device_id,
-                total_downloaded=self.total_downloaded + amount_downloaded,
-                chunk_downloaded=chunk_downloaded),
-               pickle.HIGHEST_PROTOCOL)
+            self.content = pickle.dumps(
+                BackupResults(
+                    scan_id=self.scan_id,
+                    device_id=self.device_id,
+                    total_downloaded=self.total_downloaded + amount_downloaded,
+                    chunk_downloaded=chunk_downloaded,
+                ),
+                pickle.HIGHEST_PROTOCOL,
+            )
             self.send_message_to_sink()
 
             # if amount_downloaded == total:
@@ -104,7 +116,7 @@ class BackupFilesWorker(WorkerInPublishPullPipeline, FileCopy):
         mdata_exceptions = None
 
         if not (data.move_succeeded and data.do_backup):
-            backup_full_file_name = ''
+            backup_full_file_name = ""
         else:
             self.total_reached = False
 
@@ -119,8 +131,11 @@ class BackupFilesWorker(WorkerInPublishPullPipeline, FileCopy):
             if not os.path.isdir(dest_dir):
                 # create the subfolders on the backup path
                 try:
-                    logging.debug("Creating subfolder %s on backup device %s...",
-                                  dest_dir, self.device_name)
+                    logging.debug(
+                        "Creating subfolder %s on backup device %s...",
+                        dest_dir,
+                        self.device_name,
+                    )
                     os.makedirs(dest_dir)
                     logging.debug("...backup subfolder created")
                 except (OSError, PermissionError, FileNotFoundError) as inst:
@@ -129,16 +144,19 @@ class BackupFilesWorker(WorkerInPublishPullPipeline, FileCopy):
                     # takes to query and the time it takes to create a
                     # new directory. Ignore that error.
                     if inst.errno != errno.EEXIST:
-                        logging.error("Failed to create backup subfolder: %s",
-                                      rpd_file.download_path)
+                        logging.error(
+                            "Failed to create backup subfolder: %s",
+                            rpd_file.download_path,
+                        )
                         logging.error(inst)
 
                         self.problems.append(
                             BackupSubfolderCreationProblem(
                                 folder=make_href(
-                                    name=rpd_file.download_subfolder, uri=get_uri(path=dest_dir)
+                                    name=rpd_file.download_subfolder,
+                                    uri=get_uri(path=dest_dir),
                                 ),
-                                exception=inst
+                                exception=inst,
                             )
                         )
 
@@ -151,53 +169,71 @@ class BackupFilesWorker(WorkerInPublishPullPipeline, FileCopy):
                     date = dt.strftime("%x")
                     time = dt.strftime("%X")
                 except Exception:
-                    logging.error("Could not determine the file modification time of %s",
-                                  backup_full_file_name)
-                    date = time = ''
+                    logging.error(
+                        "Could not determine the file modification time of %s",
+                        backup_full_file_name,
+                    )
+                    date = time = ""
 
                 source = rpd_file.get_souce_href()
-                device = make_href(name=rpd_file.device_display_name, uri=rpd_file.device_uri)
+                device = make_href(
+                    name=rpd_file.device_display_name, uri=rpd_file.device_uri
+                )
 
                 if data.backup_duplicate_overwrite:
-                    self.problems.append(BackupOverwrittenProblem(
-                        file_type_capitalized=rpd_file.title_capitalized,
-                        file_type=rpd_file.title,
-                        name=rpd_file.download_name,
-                        uri=get_uri(full_file_name=backup_full_file_name),
-                        source=source,
-                        device=device,
-                        date=date,
-                        time=time
-                    ))
+                    self.problems.append(
+                        BackupOverwrittenProblem(
+                            file_type_capitalized=rpd_file.title_capitalized,
+                            file_type=rpd_file.title,
+                            name=rpd_file.download_name,
+                            uri=get_uri(full_file_name=backup_full_file_name),
+                            source=source,
+                            device=device,
+                            date=date,
+                            time=time,
+                        )
+                    )
                     msg = "Overwriting backup file %s" % backup_full_file_name
                 else:
-                    self.problems.append(BackupAlreadyExistsProblem(
-                        file_type_capitalized=rpd_file.title_capitalized,
-                        file_type=rpd_file.title,
-                        name=rpd_file.download_name,
-                        uri=get_uri(full_file_name=backup_full_file_name),
-                        source=source,
-                        device=device,
-                        date=date,
-                        time=time
-                    ))
-                    msg = "Skipping backup of file %s because it already exists" % \
-                          backup_full_file_name
+                    self.problems.append(
+                        BackupAlreadyExistsProblem(
+                            file_type_capitalized=rpd_file.title_capitalized,
+                            file_type=rpd_file.title,
+                            name=rpd_file.download_name,
+                            uri=get_uri(full_file_name=backup_full_file_name),
+                            source=source,
+                            device=device,
+                            date=date,
+                            time=time,
+                        )
+                    )
+                    msg = (
+                        "Skipping backup of file %s because it already exists"
+                        % backup_full_file_name
+                    )
                 logging.warning(msg)
 
             if not backup_already_exists or data.backup_duplicate_overwrite:
-                logging.debug("Backing up file %s on device %s...",
-                              data.download_count, self.device_name)
+                logging.debug(
+                    "Backing up file %s on device %s...",
+                    data.download_count,
+                    self.device_name,
+                )
                 source = rpd_file.download_full_file_name
                 destination = backup_full_file_name
-                backup_succeeded = self.copy_from_filesystem(source, destination, rpd_file)
+                backup_succeeded = self.copy_from_filesystem(
+                    source, destination, rpd_file
+                )
                 if backup_succeeded and self.verify_file:
                     md5 = hashlib.md5(open(backup_full_file_name).read()).hexdigest()
                     if md5 != rpd_file.md5:
                         pass
                 if backup_succeeded:
-                    logging.debug("...backing up file %s on device %s succeeded",
-                                  data.download_count, self.device_name)
+                    logging.debug(
+                        "...backing up file %s on device %s succeeded",
+                        data.download_count,
+                        self.device_name,
+                    )
 
                 if backup_succeeded:
                     mdata_exceptions = copy_file_metadata(
@@ -211,48 +247,62 @@ class BackupFilesWorker(WorkerInPublishPullPipeline, FileCopy):
             else:
                 # backup any THM, audio or XMP files
                 if rpd_file.download_thm_full_name:
-                    self.backup_associate_file(dest_dir, rpd_file.download_thm_full_name)
+                    self.backup_associate_file(
+                        dest_dir, rpd_file.download_thm_full_name
+                    )
                 if rpd_file.download_audio_full_name:
-                    self.backup_associate_file(dest_dir, rpd_file.download_audio_full_name)
+                    self.backup_associate_file(
+                        dest_dir, rpd_file.download_audio_full_name
+                    )
                 if rpd_file.download_xmp_full_name:
-                    self.backup_associate_file(dest_dir, rpd_file.download_xmp_full_name)
+                    self.backup_associate_file(
+                        dest_dir, rpd_file.download_xmp_full_name
+                    )
                 if rpd_file.download_log_full_name:
-                    self.backup_associate_file(dest_dir, rpd_file.download_log_full_name)
+                    self.backup_associate_file(
+                        dest_dir, rpd_file.download_log_full_name
+                    )
 
         self.total_downloaded += rpd_file.size
         bytes_not_downloaded = rpd_file.size - self.amount_downloaded
         if bytes_not_downloaded and data.do_backup:
             self.content = pickle.dumps(
                 BackupResults(
-                    scan_id=self.scan_id, device_id=self.device_id,
-                    total_downloaded=self.total_downloaded, chunk_downloaded=bytes_not_downloaded
+                    scan_id=self.scan_id,
+                    device_id=self.device_id,
+                    total_downloaded=self.total_downloaded,
+                    chunk_downloaded=bytes_not_downloaded,
                 ),
-                pickle.HIGHEST_PROTOCOL
+                pickle.HIGHEST_PROTOCOL,
             )
             self.send_message_to_sink()
 
         self.content = pickle.dumps(
             BackupResults(
-                scan_id=self.scan_id, device_id=self.device_id, backup_succeeded=backup_succeeded,
-                do_backup=data.do_backup, rpd_file=rpd_file,
-                backup_full_file_name=backup_full_file_name, mdata_exceptions=mdata_exceptions
+                scan_id=self.scan_id,
+                device_id=self.device_id,
+                backup_succeeded=backup_succeeded,
+                do_backup=data.do_backup,
+                rpd_file=rpd_file,
+                backup_full_file_name=backup_full_file_name,
+                mdata_exceptions=mdata_exceptions,
             ),
-            pickle.HIGHEST_PROTOCOL
+            pickle.HIGHEST_PROTOCOL,
         )
         self.send_message_to_sink()
 
     def reset_problems(self) -> None:
-        self.problems = BackingUpProblems(
-            name=self.device_name, uri=self.uri
-        )
+        self.problems = BackingUpProblems(name=self.device_name, uri=self.uri)
 
     def send_problems(self) -> None:
         if self.problems:
             self.content = pickle.dumps(
                 BackupResults(
-                    scan_id=self.scan_id, device_id=self.device_id, problems=self.problems
+                    scan_id=self.scan_id,
+                    device_id=self.device_id,
+                    problems=self.problems,
                 ),
-                pickle.HIGHEST_PROTOCOL
+                pickle.HIGHEST_PROTOCOL,
             )
             self.send_message_to_sink()
             self.reset_problems()
@@ -275,7 +325,7 @@ class BackupFilesWorker(WorkerInPublishPullPipeline, FileCopy):
 
             self.check_for_command(directive, content)
 
-            data = pickle.loads(content) # type: BackupFileData
+            data = pickle.loads(content)  # type: BackupFileData
             if data.message == BackupStatus.backup_started:
                 self.reset_problems()
             elif data.message == BackupStatus.backup_completed:
@@ -288,4 +338,4 @@ class BackupFilesWorker(WorkerInPublishPullPipeline, FileCopy):
 
 
 if __name__ == "__main__":
-    backup = BackupFilesWorker()
\ No newline at end of file
+    backup = BackupFilesWorker()
