@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2015-2018 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2015-2022 Damon Lynch <damonlynch@gmail.com>
 # Copyright (C) 2012-2015 Jim Easterbrook <jim@jim-easterbrook.me.uk>
 
 # This file is part of Rapid Photo Downloader.
@@ -19,84 +19,77 @@
 # along with Rapid Photo Downloader.  If not,
 # see <http://www.gnu.org/licenses/>.
 
-__author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2015-2018, Damon Lynch. Copyright 2012-2015 Jim Easterbrook."
+__author__ = "Damon Lynch"
+__copyright__ = "Copyright 2015-2022, Damon Lynch. Copyright 2012-2015 Jim Easterbrook."
 
+import io
 import logging
 import os
-import io
-from collections import namedtuple
 import re
-from typing import Optional, List, Tuple
+from typing import Optional, List, Tuple, Union
 
 import gphoto2 as gp
-from raphodo.storage import StorageSpace
+from raphodo.storage.storage import StorageSpace, udev_attributes
 from raphodo.constants import CameraErrorCode
 from raphodo.utilities import format_size_for_user
+from raphodo.cameraerror import CameraError, CameraProblemEx
 
 
 def python_gphoto2_version():
-    return  gp.__version__
+    return gp.__version__
+
 
 def gphoto2_version():
     return gp.gp_library_version(0)[0]
 
 
+def gphoto2_python_logging():
+    """
+    Version 2.0.0 of gphoto2 introduces a COMPATIBILITY CHANGE:
+    gp_log_add_func and use_python_logging now return a
+    Python object which must be stored until logging is no longer needed.
+    Could just go with the None returned by default from a function that
+    returns nothing, but want to make this explicit.
+
+    :return: either True or a Python object that must be stored until logging
+     is no longer needed
+    """
+
+    return gp.use_python_logging() or True
+
+
+def autodetect_cameras(
+    context: gp.Context, suppress_errors: bool = True
+) -> Union[gp.CameraList, List]:
+    """
+    Do camera auto detection for multiple versions of gphoto2-python
+
+    Version 2.2.0 of gphoto2 introduces a COMPATIBILITY CHANGE:
+    Removed Context.camera_autodetect method.
+    Was quickly reintroduced in 2.2.1, but is due for removal.
+
+    :return: CameraList of model and port
+    """
+
+    try:
+        return gp.check_result(gp.gp_camera_autodetect(context))
+    except Exception:
+        if not suppress_errors:
+            raise
+        return []
+
+
 # convert error codes to error names
 gphoto2_error_codes = {
-    code: name for code, name in (
-        ((getattr(gp, attr), attr) for attr in dir(gp) if attr.startswith('GP_ERROR'))
+    code: name
+    for code, name in (
+        ((getattr(gp, attr), attr) for attr in dir(gp) if attr.startswith("GP_ERROR"))
     )
 }
 
 
 def gphoto2_named_error(code: int) -> str:
-    return gphoto2_error_codes.get(code, 'Unknown gphoto2 error')
-
-
-class CameraError(Exception):
-    def __init__(self, code: CameraErrorCode) -> None:
-        self.code = code
-
-    def __repr__(self) -> str:
-        if self.code == CameraErrorCode.inaccessible:
-            return "inaccessible"
-        else:
-            return "locked"
-
-    def __str__(self) -> str:
-        if self.code == CameraErrorCode.inaccessible:
-            return "The camera is inaccessible"
-        else:
-            return "The camera is locked"
-
-
-class CameraProblemEx(CameraError):
-    def __init__(self, code: CameraErrorCode,
-                 gp_exception: Optional[gp.GPhoto2Error]=None,
-                 py_exception: Optional[Exception]=None) -> None:
-        super().__init__(code)
-        if gp_exception is not None:
-            self.gp_code = gp_exception.code
-        else:
-            self.gp_code = None
-        self.py_exception = py_exception
-
-    def __repr__(self) -> str:
-        if self.code == CameraErrorCode.read:
-            return "read error"
-        elif self.code == CameraErrorCode.write:
-            return 'write error'
-        else:
-            return repr(super())
-
-    def __str__(self) -> str:
-        if self.code == CameraErrorCode.read:
-            return "Could not read file from camera"
-        elif self.code == CameraErrorCode.write:
-            return 'Could not write file from camera'
-        else:
-            return str(super())
+    return gphoto2_error_codes.get(code, "Unknown gphoto2 error")
 
 
 def generate_devname(camera_port: str) -> Optional[str]:
@@ -112,27 +105,42 @@ def generate_devname(camera_port: str) -> Optional[str]:
     :return: devname if it could be generated, else None
     """
 
-    match = re.match('usb:([0-9]+),([0-9]+)', camera_port)
+    match = re.match("usb:([0-9]+),([0-9]+)", camera_port)
     if match is not None:
         p1, p2 = match.groups()
-        return '/dev/bus/usb/{}/{}'.format(p1, p2)
+        return "/dev/bus/usb/{}/{}".format(p1, p2)
     return None
 
 
+def camera_is_mtp_device(camera_port: str) -> bool:
+    devname = generate_devname(camera_port)
+    if devname is not None:
+        udev_attr = udev_attributes(devname)
+        if udev_attr is not None:
+            return udev_attr.is_mtp_device
+    logging.error("Could not determine udev values for camera at port %s", camera_port)
+    return False
+
+
 class Camera:
 
     """Access a camera via libgphoto2."""
 
-    def __init__(self, model: str,
-                 port:str,
-                 get_folders: bool=True,
-                 raise_errors: bool=False,
-                 context: gp.Context=None,
-                 specific_folders: Optional[List[str]]=None) -> None:
+    def __init__(
+        self,
+        model: str,
+        port: str,
+        is_mtp_device: bool,
+        get_folders: bool = True,
+        raise_errors: bool = False,
+        context: gp.Context = None,
+        specific_folders: Optional[List[str]] = None,
+    ) -> None:
         """
         Initialize a camera via libgphoto2.
 
-        :param model: camera model, as returned by camera_autodetect()
+        :param model: camera model, as returned by camera_autodetect() or
+         gp_camera_autodetect()
         :param port: camera port, as returned by camera_autodetect()
         :param get_folders: whether to detect the DCIM folders on the
          camera
@@ -146,6 +154,7 @@ class Camera:
 
         self.model = model
         self.port = port
+        self.is_mtp_device = is_mtp_device
         # class method _concise_model_name discusses why a display name is
         # needed
         self.display_name = model
@@ -158,7 +167,7 @@ class Camera:
 
         self._select_camera(model, port)
 
-        self.specific_folders = None # type: List[str]
+        self.specific_folders = None  # type: Optional[List[str]]
         self.specific_folder_located = False
         self._dual_slots_active = False
 
@@ -170,13 +179,20 @@ class Camera:
             self.camera_initialized = True
         except gp.GPhoto2Error as e:
             if e.code == gp.GP_ERROR_IO_USB_CLAIM:
+                error_code = CameraErrorCode.inaccessible
                 logging.error("{} is already mounted".format(model))
             elif e.code == gp.GP_ERROR:
-                logging.error("An error occurred initializing the camera using libgphoto2")
+                logging.error(
+                    "An error occurred initializing the camera using libgphoto2"
+                )
+                error_code = CameraErrorCode.inaccessible
             else:
-                logging.error("Unable to access camera: %s", gphoto2_named_error(e.code))
+                logging.error(
+                    "Unable to access camera: %s", gphoto2_named_error(e.code)
+                )
+                error_code = CameraErrorCode.locked
             if raise_errors:
-                raise CameraProblemEx(CameraErrorCode.inaccessible, gp_exception=e)
+                raise CameraProblemEx(error_code, gp_exception=e)
             return
 
         concise_model_name = self._concise_model_name()
@@ -186,13 +202,20 @@ class Camera:
         if get_folders:
             try:
                 self.specific_folders = self._locate_specific_folders(
-                    path='/', specific_folders=specific_folders
+                    path="/", specific_folders=specific_folders
                 )
                 self.specific_folder_located = len(self.specific_folders) > 0
+
+                logging.debug(
+                    "Folders located on %s: %s",
+                    self.display_name,
+                    ", ".join(", ".join(map(str, sl)) for sl in self.specific_folders),
+                )
             except gp.GPhoto2Error as e:
                 logging.error(
                     "Unable to access camera %s: %s. Is it locked?",
-                    self.display_name, gphoto2_named_error(e.code)
+                    self.display_name,
+                    gphoto2_named_error(e.code),
                 )
                 if raise_errors:
                     self.free_camera()
@@ -202,31 +225,49 @@ class Camera:
         self.audio_files = {}
         self.video_thumbnails = []
         abilities = self.camera.get_abilities()
-        self.can_fetch_thumbnails = abilities.file_operations & gp.GP_FILE_OPERATION_PREVIEW != 0
+        self.can_fetch_thumbnails = (
+            abilities.file_operations & gp.GP_FILE_OPERATION_PREVIEW != 0
+        )
 
-    def camera_has_dcim_like_folder(self) -> bool:
+    def camera_has_folders_to_scan(self) -> bool:
         """
-        Check whether the camera has been initialized and if a DCIM or other specific folder
-        has been located
+        Check whether the camera has been initialized and if a DCIM or other specific
+        folder has been located
 
-        :return: True if the camera is initialized and a DCIM or other specific folder has
-                 been located
+        :return: True if the camera is initialized and a DCIM or other specific folder
+        has been located
         """
         return self.camera_initialized and self.specific_folder_located
 
-    def _locate_specific_folders(self,
-                     path: str,
-                     specific_folders: Optional[List[str]]) -> List[Optional[List[str]]]:
+    @staticmethod
+    def _locate_specific_subfolders(subfolders, subpath, specific_folders):
+        return [
+            os.path.join(subpath, folder)
+            for folder in specific_folders
+            if folder in subfolders
+        ]
+
+    def _locate_specific_folders(
+        self, path: str, specific_folders: Optional[List[str]]
+    ) -> List[Optional[List[str]]]:
         """
         Scan camera looking for folders such as DCIM,  PRIVATE, and MP_ROOT.
 
-        Looks in either the root of the path passed, or in one of the root
-        folders subfolders (it does not scan subfolders of those subfolders).
+        For MTP devices, looks in either the root of the path passed, or in one of the
+        root folders subfolders (it does not scan subfolders of those subfolders)
+
+        For PTP devices, also look into subfolders of the subfolders, e.g. not just
+        /store_00020001/DCIM , but also /store_10000001/SLOT 1/DCIM
 
         Returns all instances of the specific folders, which is helpful for
         cameras that have more than one storage (memory card / internal memory)
         slot.
 
+        Returns a list of lists:
+        1. Top level list is length 2 if camera has two memory card slots and
+           they are both have a DCIM folder
+        2. Else top level will be length 1 (or 0 if empty)
+
         No error checking: exceptions must be caught by the caller
 
         :param path: the root folder to start scanning in
@@ -243,18 +284,42 @@ class Camera:
             found_folders = [[path + folder] for folder in folders]
         else:
             found_folders = []
-
             # look for the folders one level down from the root folder
             # it is at this level that specific folders like DCIM will be found
             for subfolder in folders:
                 subpath = os.path.join(path, subfolder)
-                subfolders = dict(self.camera.folder_list_folders(subpath, self.context))
-                ff = [
-                    os.path.join(subpath, folder) for folder in specific_folders
-                    if folder in subfolders
-                ]
+                subfolders = dict(
+                    self.camera.folder_list_folders(subpath, self.context)
+                )
+                ff = self._locate_specific_subfolders(
+                    subfolders=subfolders,
+                    subpath=subpath,
+                    specific_folders=specific_folders,
+                )
                 if ff:
                     found_folders.append(ff)
+                elif not self.is_mtp_device:
+                    # look at subfolders of subfolders, e.g. Fujifilm dual slot cameras
+                    # which use "SLOT 1" and "SLOT 2":
+                    # /store_10000001/SLOT 1/DCIM
+                    # /store_10000001/SLOT 2/DCIM
+                    found_subfolders = []
+                    for nested_subfolder in subfolders:
+                        nested_subpath = os.path.join(subpath, nested_subfolder)
+                        nested_subfolders = dict(
+                            self.camera.folder_list_folders(
+                                nested_subpath, self.context
+                            )
+                        )
+                        ff = self._locate_specific_subfolders(
+                            subfolders=nested_subfolders,
+                            subpath=nested_subpath,
+                            specific_folders=specific_folders,
+                        )
+                        if ff:
+                            found_subfolders.extend(ff)
+                    if found_subfolders:
+                        found_folders.append(found_subfolders)
 
         self._dual_slots_active = len(found_folders) > 1
 
@@ -275,10 +340,10 @@ class Camera:
         size = info.file.size
         return modification_time, size
 
-    def get_exif_extract(self, folder: str,
-                         file_name: str,
-                         size_in_bytes: int=200) -> bytearray:
-        """"
+    def get_exif_extract(
+        self, folder: str, file_name: str, size_in_bytes: int = 200
+    ) -> bytearray:
+        """ "
         Attempt to read only the exif portion of the file.
 
         Assumes exif is located at the beginning of the file.
@@ -300,7 +365,8 @@ class Camera:
         except gp.GPhoto2Error as e:
             logging.error(
                 "Unable to extract portion of file from camera %s: %s",
-                self.display_name, gphoto2_named_error(e.code)
+                self.display_name,
+                gphoto2_named_error(e.code),
             )
             raise CameraProblemEx(code=CameraErrorCode.read, gp_exception=e)
         else:
@@ -328,14 +394,17 @@ class Camera:
             exif_data = gp.check_result(gp.gp_file_get_data_and_size(camera_file))
         except gp.GPhoto2Error as ex:
             logging.error(
-                'Error getting exif info for %s from camera %s: %s',
-                os.path.join(folder, file_name), self.display_name, gphoto2_named_error(ex.code)
+                "Error getting exif info for %s from camera %s: %s",
+                os.path.join(folder, file_name),
+                self.display_name,
+                gphoto2_named_error(ex.code),
             )
             raise CameraProblemEx(code=CameraErrorCode.read, gp_exception=ex)
         return bytearray(exif_data)
 
-    def get_exif_extract_from_jpeg_manual_parse(self, folder: str,
-                                            file_name: str) -> Optional[bytes]:
+    def get_exif_extract_from_jpeg_manual_parse(
+        self, folder: str, file_name: str
+    ) -> Optional[bytes]:
         """
         Extract exif section of a jpeg.
 
@@ -366,27 +435,30 @@ class Camera:
 
         view = memoryview(bytearray(read0_size))
         try:
-            bytes_read = gp.check_result(self.camera.file_read(
-                folder, file_name, gp.GP_FILE_TYPE_NORMAL,
-                0, view, self.context))
+            bytes_read = gp.check_result(
+                self.camera.file_read(
+                    folder, file_name, gp.GP_FILE_TYPE_NORMAL, 0, view, self.context
+                )
+            )
         except gp.GPhoto2Error as ex:
             logging.error(
-                'Error reading %s from camera: %s',
-                os.path.join(folder, file_name), gphoto2_named_error(ex.code)
+                "Error reading %s from camera: %s",
+                os.path.join(folder, file_name),
+                gphoto2_named_error(ex.code),
             )
             return None
 
         jpeg_header = view.tobytes()
         view.release()
 
-        if jpeg_header[0:2] != b'\xff\xd8':
+        if jpeg_header[0:2] != b"\xff\xd8":
             logging.error("%s not a jpeg image: no SOI marker", file_name)
             return None
 
         app_marker = jpeg_header[2:4]
 
         # Step 2: handle presence of APP0 - it's optional
-        if app_marker == b'\xff\xe0':
+        if app_marker == b"\xff\xe0":
             # There is an APP0 before the probable APP1
             # Don't neeed the content of the APP0
             app0_data_length = jpeg_header[4] * 256 + jpeg_header[5]
@@ -396,29 +468,37 @@ class Camera:
             read1_size = app0_data_length + 2
             app0_view = memoryview(bytearray(read1_size))
             try:
-                bytes_read = gp.check_result(self.camera.file_read(
-                    folder, file_name, gp.GP_FILE_TYPE_NORMAL,
-                    read0_size, app0_view, self.context))
+                bytes_read = gp.check_result(
+                    self.camera.file_read(
+                        folder,
+                        file_name,
+                        gp.GP_FILE_TYPE_NORMAL,
+                        read0_size,
+                        app0_view,
+                        self.context,
+                    )
+                )
             except gp.GPhoto2Error as ex:
                 logging.error(
-                    'Error reading %s from camera: %s',
-                    os.path.join(folder, file_name), gphoto2_named_error(ex.code)
+                    "Error reading %s from camera: %s",
+                    os.path.join(folder, file_name),
+                    gphoto2_named_error(ex.code),
                 )
             app0 = app0_view.tobytes()
             app0_view.release()
-            app_marker = app0[(exif_header_length + 2) * -1:exif_header_length * -1]
-            exif_header = app0[exif_header_length * -1:]
+            app_marker = app0[(exif_header_length + 2) * -1 : exif_header_length * -1]
+            exif_header = app0[exif_header_length * -1 :]
             jpeg_header = jpeg_header + app0
             offset = read0_size + read1_size
         else:
-            exif_header = jpeg_header[exif_header_length * -1:]
+            exif_header = jpeg_header[exif_header_length * -1 :]
             offset = read0_size
 
         # Step 3: process exif header
-        if app_marker != b'\xff\xe1':
+        if app_marker != b"\xff\xe1":
             logging.error("Could not locate APP1 marker in %s", file_name)
             return None
-        if exif_header[2:6] != b'Exif' or exif_header[6:8] != b'\x00\x00':
+        if exif_header[2:6] != b"Exif" or exif_header[6:8] != b"\x00\x00":
             logging.error("APP1 is malformed in %s", file_name)
             return None
         app1_data_length = exif_header[0] * 256 + exif_header[1]
@@ -428,30 +508,43 @@ class Camera:
         try:
             bytes_read = gp.check_result(
                 self.camera.file_read(
-                    folder, file_name, gp.GP_FILE_TYPE_NORMAL, offset, view, self.context
+                    folder,
+                    file_name,
+                    gp.GP_FILE_TYPE_NORMAL,
+                    offset,
+                    view,
+                    self.context,
                 )
             )
         except gp.GPhoto2Error as ex:
             logging.error(
-                'Error reading %s from camera: %s',
-                os.path.join(folder, file_name), gphoto2_named_error(ex.code)
+                "Error reading %s from camera: %s",
+                os.path.join(folder, file_name),
+                gphoto2_named_error(ex.code),
             )
             return None
         return jpeg_header + view.tobytes()
 
-    def _get_file(self, dir_name: str,
-                  file_name: str,
-                  dest_full_filename: Optional[str]=None,
-                  file_type: int=gp.GP_FILE_TYPE_NORMAL) -> gp.CameraFile:
+    def _get_file(
+        self,
+        dir_name: str,
+        file_name: str,
+        dest_full_filename: Optional[str] = None,
+        file_type: int = gp.GP_FILE_TYPE_NORMAL,
+    ) -> gp.CameraFile:
 
         try:
             camera_file = gp.check_result(
-                gp.gp_camera_file_get(self.camera, dir_name, file_name, file_type, self.context)
+                gp.gp_camera_file_get(
+                    self.camera, dir_name, file_name, file_type, self.context
+                )
             )
         except gp.GPhoto2Error as ex:
             logging.error(
-                'Error reading %s from camera %s: %s',
-                os.path.join(dir_name, file_name), self.display_name, gphoto2_named_error(ex.code)
+                "Error reading %s from camera %s: %s",
+                os.path.join(dir_name, file_name),
+                self.display_name,
+                gphoto2_named_error(ex.code),
             )
             raise CameraProblemEx(code=CameraErrorCode.read, gp_exception=ex)
 
@@ -460,17 +553,16 @@ class Camera:
                 gp.check_result(gp.gp_file_save(camera_file, dest_full_filename))
             except gp.GPhoto2Error as ex:
                 logging.error(
-                    'Error saving %s from camera %s: %s',
-                    os.path.join(dir_name, file_name), self.display_name,
-                    gphoto2_named_error(ex.code)
+                    "Error saving %s from camera %s: %s",
+                    os.path.join(dir_name, file_name),
+                    self.display_name,
+                    gphoto2_named_error(ex.code),
                 )
                 raise CameraProblemEx(code=CameraErrorCode.write, gp_exception=ex)
 
         return camera_file
 
-    def save_file(self, dir_name: str,
-                  file_name: str,
-                  dest_full_filename: str) -> None:
+    def save_file(self, dir_name: str, file_name: str, dest_full_filename: str) -> None:
         """
         Save the file from the camera to a local destination.
 
@@ -482,11 +574,14 @@ class Camera:
 
         self._get_file(dir_name, file_name, dest_full_filename)
 
-    def save_file_chunk(self, dir_name: str,
-                        file_name: str,
-                        chunk_size_in_bytes: int,
-                        dest_full_filename: str,
-                        mtime: int=None) -> None:
+    def save_file_chunk(
+        self,
+        dir_name: str,
+        file_name: str,
+        chunk_size_in_bytes: int,
+        dest_full_filename: str,
+        mtime: int = None,
+    ) -> None:
         """
         Save the file from the camera to a local destination.
 
@@ -505,7 +600,7 @@ class Camera:
         view = memoryview(buffer)
         dest_file = None
         try:
-            dest_file = io.open(dest_full_filename, 'wb')
+            dest_file = io.open(dest_full_filename, "wb")
             src_bytes = view.tobytes()
             dest_file.write(src_bytes)
             dest_file.close()
@@ -513,21 +608,26 @@ class Camera:
                 os.utime(dest_full_filename, times=(mtime, mtime))
         except (OSError, PermissionError) as ex:
             logging.error(
-                'Error saving file %s from camera %s: %s',
-                os.path.join(dir_name, file_name), self.display_name, gphoto2_named_error(ex.errno)
+                "Error saving file %s from camera %s: %s",
+                os.path.join(dir_name, file_name),
+                self.display_name,
+                gphoto2_named_error(ex.errno),
             )
             if dest_file is not None:
                 dest_file.close()
             raise CameraProblemEx(code=CameraErrorCode.write, py_exception=ex)
 
-    def save_file_by_chunks(self, dir_name: str,
-                            file_name: str,
-                            size: int,
-                            dest_full_filename: str,
-                            progress_callback,
-                            check_for_command,
-                            return_file_bytes = False,
-                            chunk_size=1048576) -> Optional[bytes]:
+    def save_file_by_chunks(
+        self,
+        dir_name: str,
+        file_name: str,
+        size: int,
+        dest_full_filename: str,
+        progress_callback,
+        check_for_command,
+        return_file_bytes=False,
+        chunk_size=1048576,
+    ) -> Optional[bytes]:
         """
         :param dir_name: directory on the camera
         :param file_name: the photo or video
@@ -555,8 +655,12 @@ class Camera:
             try:
                 bytes_read = gp.check_result(
                     self.camera.file_read(
-                        dir_name, file_name, gp.GP_FILE_TYPE_NORMAL, offset, view[offset:stop],
-                        self.context
+                        dir_name,
+                        file_name,
+                        gp.GP_FILE_TYPE_NORMAL,
+                        offset,
+                        view[offset:stop],
+                        self.context,
                     )
                 )
                 amount_downloaded += bytes_read
@@ -564,9 +668,10 @@ class Camera:
                     progress_callback(amount_downloaded, size)
             except gp.GPhoto2Error as ex:
                 logging.error(
-                    'Error copying file %s from camera %s: %s',
-                    os.path.join(dir_name, file_name), self.display_name,
-                    gphoto2_named_error(ex.code)
+                    "Error copying file %s from camera %s: %s",
+                    os.path.join(dir_name, file_name),
+                    self.display_name,
+                    gphoto2_named_error(ex.code),
                 )
                 if progress_callback is not None:
                     progress_callback(size, size)
@@ -574,14 +679,17 @@ class Camera:
 
         dest_file = None
         try:
-            dest_file = io.open(dest_full_filename, 'wb')
+            dest_file = io.open(dest_full_filename, "wb")
             src_bytes = view.tobytes()
             dest_file.write(src_bytes)
             dest_file.close()
         except (OSError, PermissionError) as ex:
             logging.error(
-                'Error saving file %s from camera %s. Error %s: %s',
-                os.path.join(dir_name, file_name), self.display_name, ex.errno, ex.strerror
+                "Error saving file %s from camera %s. Error %s: %s",
+                os.path.join(dir_name, file_name),
+                self.display_name,
+                ex.errno,
+                ex.strerror,
             )
             if dest_file is not None:
                 dest_file.close()
@@ -590,10 +698,13 @@ class Camera:
         if return_file_bytes:
             return src_bytes
 
-    def get_thumbnail(self, dir_name: str,
-                      file_name: str,
-                      ignore_embedded_thumbnail=False,
-                      cache_full_filename: Optional[str]=None) -> Optional[bytes]:
+    def get_thumbnail(
+        self,
+        dir_name: str,
+        file_name: str,
+        ignore_embedded_thumbnail=False,
+        cache_full_filename: Optional[str] = None,
+    ) -> Optional[bytes]:
         """
         :param dir_name: directory on the camera
         :param file_name: the photo or video
@@ -618,9 +729,10 @@ class Camera:
             thumbnail_data = gp.check_result(gp.gp_file_get_data_and_size(camera_file))
         except gp.GPhoto2Error as ex:
             logging.error(
-                'Error getting image %s from camera %s: %s',
-                os.path.join(dir_name, file_name), self.display_name,
-                gphoto2_named_error(ex.code)
+                "Error getting image %s from camera %s: %s",
+                os.path.join(dir_name, file_name),
+                self.display_name,
+                gphoto2_named_error(ex.code),
             )
             raise CameraProblemEx(code=CameraErrorCode.read, gp_exception=ex)
 
@@ -641,8 +753,10 @@ class Camera:
             thumbnail_data = gp.check_result(gp.gp_file_get_data_and_size(camera_file))
         except gp.GPhoto2Error as ex:
             logging.error(
-                'Error getting THM file %s from camera %s: %s',
-                 os.path.join(dir_name, file_name), self.display_name, gphoto2_named_error(ex.code)
+                "Error getting THM file %s from camera %s: %s",
+                os.path.join(dir_name, file_name),
+                self.display_name,
+                gphoto2_named_error(ex.code),
             )
             raise CameraProblemEx(code=CameraErrorCode.read, gp_exception=ex)
 
@@ -650,7 +764,7 @@ class Camera:
             data = memoryview(thumbnail_data)
             return data.tobytes()
 
-    def _select_camera(self, model, port_name)  -> None:
+    def _select_camera(self, model, port_name) -> None:
         # Code from Jim Easterbrook's Photoini
         # initialise camera
         self.camera = gp.Camera()
@@ -684,21 +798,35 @@ class Camera:
          if not found.
         """
         if self.camera_config is None:
-            self.camera_config = self.camera.get_config(self.context)
+            try:
+                self.camera_config = self.camera.get_config(self.context)
+            except gp.GPhoto2Error as e:
+                if e.code == gp.GP_ERROR_NOT_SUPPORTED:
+                    logging.error(
+                        "Getting camera configuration not supported for %s",
+                        self.display_name,
+                    )
+                else:
+                    logging.error(
+                        "Unknown error getting camera configuration for %s",
+                        self.display_name,
+                    )
+                return ""
+
         # Here we really see the difference between C and python!
         child_count = self.camera_config.count_children()
         for i in range(child_count):
             child1 = self.camera_config.get_child(i)
             child_type = child1.get_type()
-            if child1.get_name() == 'status' and child_type == gp.GP_WIDGET_SECTION:
+            if child1.get_name() == "status" and child_type == gp.GP_WIDGET_SECTION:
                 child1_count = child1.count_children()
                 for j in range(child1_count):
                     child2 = child1.get_child(j)
-                    if child2.get_name() == 'cameramodel':
+                    if child2.get_name() == "cameramodel":
                         return child2.get_value()
-        return ''
+        return ""
 
-    def get_storage_media_capacity(self, refresh: bool=False) -> List[StorageSpace]:
+    def get_storage_media_capacity(self, refresh: bool = False) -> List[StorageSpace]:
         """
         Determine the bytes free and bytes total (media capacity)
         :param refresh: if True, get updated instead of cached values
@@ -710,20 +838,22 @@ class Camera:
         storage_capacity = []
         for media_index in range(len(self.storage_info)):
             info = self.storage_info[media_index]
-            if not (info.fields & gp.GP_STORAGEINFO_MAXCAPACITY and
-                    info.fields & gp.GP_STORAGEINFO_FREESPACEKBYTES):
-                logging.error('Could not locate storage on %s', self.display_name)
+            if not (
+                info.fields & gp.GP_STORAGEINFO_MAXCAPACITY
+                and info.fields & gp.GP_STORAGEINFO_FREESPACEKBYTES
+            ):
+                logging.error("Could not locate storage on %s", self.display_name)
             else:
                 storage_capacity.append(
                     StorageSpace(
                         bytes_free=info.freekbytes * 1024,
                         bytes_total=info.capacitykbytes * 1024,
-                        path=info.basedir
+                        path=info.basedir,
                     )
                 )
         return storage_capacity
 
-    def get_storage_descriptions(self, refresh: bool=False) -> List[str]:
+    def get_storage_descriptions(self, refresh: bool = False) -> List[str]:
         """
         Storage description is used in MTP path names by gvfs and KDE.
 
@@ -738,7 +868,7 @@ class Camera:
                 descriptions.append(info.description)
         return descriptions
 
-    def no_storage_media(self, refresh: bool=False) -> int:
+    def no_storage_media(self, refresh: bool = False) -> int:
         """
         Return the number of storage media (e.g. memory cards) the
         camera has
@@ -751,8 +881,7 @@ class Camera:
     def _get_storage_info(self, refresh: bool):
         """
         Load the gphoto2 storage information
-        :param refresh: if True, refresh the storage information, i.e.
-         load it
+        :param refresh: if True, refresh the storage information, i.e. load it
         """
         if not self.storage_info or refresh:
             try:
@@ -760,7 +889,8 @@ class Camera:
             except gp.GPhoto2Error as e:
                 logging.error(
                     "Unable to determine storage info for camera %s: %s",
-                    self.display_name, gphoto2_named_error(e.code)
+                    self.display_name,
+                    gphoto2_named_error(e.code),
                 )
                 self.storage_info = []
 
@@ -774,13 +904,13 @@ class Camera:
         if self.specific_folders is None:
             logging.warning(
                 "dual_slots_active() called before camera's folders scanned for %s",
-                self.display_name
+                self.display_name,
             )
             return False
         if not self.specific_folder_located:
             logging.warning(
                 "dual_slots_active() called when no specific folders found for %s",
-                self.display_name
+                self.display_name,
             )
             return False
         return self.no_storage_media() > 1 and self._dual_slots_active
@@ -794,38 +924,45 @@ class Camera:
         :return: True if unlocked, else False
         """
         try:
-            self.camera.folder_list_folders('/', self.context)
+            self.camera.folder_list_folders("/", self.context)
         except gp.GPhoto2Error as e:
             logging.error(
                 "Unable to access camera %s: %s. Is it locked?",
-                self.display_name, gphoto2_named_error(e.code)
+                self.display_name,
+                gphoto2_named_error(e.code),
             )
             return False
         else:
             return True
 
 
-
 def dump_camera_details() -> None:
     import itertools
+
     context = gp.Context()
-    cameras = context.camera_autodetect()
+    cameras = autodetect_cameras(context)
     for model, port in cameras:
-        c = Camera(model=model, port=port, context=context)
+        is_mtp_device = camera_is_mtp_device(camera_port=port)
+        c = Camera(
+            model=model,
+            port=port,
+            is_mtp_device=is_mtp_device,
+            context=context,
+        )
         if not c.camera_initialized:
             logging.error("Camera %s could not be initialized", model)
         else:
             print()
             print(c.display_name)
-            print('=' * len(c.display_name))
+            print("=" * len(c.display_name))
+            print(f"\nMTP: {is_mtp_device}")
             print()
             if not c.specific_folder_located:
-                print("Speicifc folder was not located")
+                print("Specific folder was not located")
             else:
                 print(
-                    "Specific folders:", ', '.join(
-                        itertools.chain.from_iterable(c.specific_folders)
-                    )
+                    "Specific folders:",
+                    ", ".join(itertools.chain.from_iterable(c.specific_folders)),
                 )
                 print("Can fetch thumbnails:", c.can_fetch_thumbnails)
 
@@ -833,52 +970,56 @@ def dump_camera_details() -> None:
                 if not sc:
                     print("Unable to determine storage media capacity")
                 else:
-                    title = 'Storage capacity'
-                    print('\n{}\n{}'.format(title, '-' * len(title)))
+                    title = "Storage capacity"
+                    print("\n{}\n{}".format(title, "-" * len(title)))
                     for ss in sc:
                         print(
-                            '\nPath: {}\nCapacity: {}\nFree {}'.format(
+                            "\nPath: {}\nCapacity: {}\nFree {}".format(
                                 ss.path,
                                 format_size_for_user(ss.bytes_total),
-                                format_size_for_user(ss.bytes_free)
+                                format_size_for_user(ss.bytes_free),
                             )
                         )
                 sd = c.get_storage_descriptions()
                 if not sd:
                     print("Unable to determine storage descriptions")
                 else:
-                    title = 'Storage description(s)'
-                    print('\n{}\n{}'.format(title, '-' * len(title)))
+                    title = "Storage description(s)"
+                    print("\n{}\n{}".format(title, "-" * len(title)))
                     for ss in sd:
-                        print('\n{}'.format(ss))
+                        print("\n{}".format(ss))
 
         c.free_camera()
 
 
 if __name__ == "__main__":
+    print("gphoto2 python: ", python_gphoto2_version())
+    # logging = gphoto2_python_logging()
 
-    if False:
+    if True:
         dump_camera_details()
 
     if True:
 
-        #Test stub
+        # Test stub
         gp_context = gp.Context()
         # Assume gphoto2 version 2.5 or greater
-        cameras = gp_context.camera_autodetect()
+        cameras = autodetect_cameras(gp_context)
         for name, value in cameras:
             camera = name
             port = value
             # print(port)
-            c = Camera(model=camera, port=port, specific_folders=['DCIM', 'MISC'])
+            is_mtp_device = camera_is_mtp_device(camera_port=port)
+            c = Camera(
+                model=camera,
+                port=port,
+                is_mtp_device=is_mtp_device,
+                specific_folders=["DCIM", "MISC"],
+            )
             # c = Camera(model=camera, port=port)
             print(c.no_storage_media(), c.dual_slots_active, c.specific_folders)
 
-            for name, value in c.camera.folder_list_files('/', c.context):
+            for name, value in c.camera.folder_list_files("/", c.context):
                 print(name, value)
 
             c.free_camera()
-
-
-
-
