@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2020 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2007-2022 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -17,17 +17,17 @@
 # along with Rapid Photo Downloader.  If not,
 # see <http://www.gnu.org/licenses/>.
 
-__author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2007-2020, Damon Lynch"
+__author__ = "Damon Lynch"
+__copyright__ = "Copyright 2007-2022, Damon Lynch"
 
-from enum import (Enum, IntEnum)
+from enum import Enum, IntEnum, auto
 from PyQt5.QtCore import Qt
 from PyQt5.QtGui import QFont, QFontMetrics, QColor
 
 PROGRAM_NAME = "Rapid Photo Downloader"
-logfile_name = 'rapid-photo-downloader.log'
+logfile_name = "rapid-photo-downloader.log"
 
-remote_versions_file = 'https://www.damonlynch.net/rapid/version.json'
+remote_versions_file = "https://damonlynch.net/rapid/version.json"
 
 # If set to True, the ability to check for a new version will be removed
 # from the user interface and disabled in program logic.
@@ -101,20 +101,28 @@ class DownloadStatus(Enum):
     download_failed = 7
 
 
-Downloaded = (DownloadStatus.downloaded,
-              DownloadStatus.downloaded_with_warning,
-              DownloadStatus.backup_problem)
+Downloaded = (
+    DownloadStatus.downloaded,
+    DownloadStatus.downloaded_with_warning,
+    DownloadStatus.backup_problem,
+)
 
 
-DownloadWarning = {DownloadStatus.downloaded_with_warning, DownloadStatus.backup_problem}
-DownloadFailure = {DownloadStatus.download_and_backup_failed, DownloadStatus.download_failed}
+DownloadWarning = {
+    DownloadStatus.downloaded_with_warning,
+    DownloadStatus.backup_problem,
+}
+DownloadFailure = {
+    DownloadStatus.download_and_backup_failed,
+    DownloadStatus.download_failed,
+}
 
 
 download_status_error_severity = {
     DownloadStatus.downloaded_with_warning: ErrorType.warning,
     DownloadStatus.backup_problem: ErrorType.serious_error,
     DownloadStatus.download_and_backup_failed: ErrorType.serious_error,
-    DownloadStatus.download_failed: ErrorType.serious_error
+    DownloadStatus.download_failed: ErrorType.serious_error,
 }
 
 
@@ -162,6 +170,12 @@ BackupFailureType = DisplayingFilesOfType
 DownloadingFileTypes = DisplayingFilesOfType
 
 
+class WindowsDriveType(IntEnum):
+    removable_disk = 2
+    local_disk = 3
+    network_drive = 4
+
+
 class DestinationDisplayType(Enum):
     folder_only = 1
     usage_only = 2
@@ -186,9 +200,10 @@ class DestinationDisplayTooltipState(Enum):
 
 
 class DeviceType(Enum):
-    camera = 1
-    volume = 2
-    path = 3
+    camera = 1  # camera accessed using PTP
+    camera_fuse = 2  # a camera-like device accessed using fuse, e.g. Apple iOS device
+    volume = 3  # a memory card or external drive, etc.
+    path = 4  # file system path
 
 
 class BackupDeviceType:
@@ -251,6 +266,12 @@ class ApplicationState(Enum):
     exiting = 2
 
 
+class PostCameraUnmountAction(Enum):
+    scan = 1
+    download = 2
+    nothing = 3
+
+
 class Show(IntEnum):
     all = 1
     new_only = 2
@@ -271,9 +292,9 @@ class JobCodeSort(IntEnum):
 
 
 Checked_Status = {
-    Qt.Checked: 'checked',
-    Qt.Unchecked: 'unchecked',
-    Qt.PartiallyChecked: 'partially checked'
+    Qt.Checked: "checked",
+    Qt.Unchecked: "unchecked",
+    Qt.PartiallyChecked: "partially checked",
 }
 
 
@@ -340,6 +361,9 @@ class CameraErrorCode(Enum):
     locked = 2
     read = 3
     write = 4
+    pair = 5
+    mount = 6
+    devicename = 7
 
 
 class ViewRowType(Enum):
@@ -360,19 +384,24 @@ class NameGenerationType(Enum):
 
 
 class CustomColors(Enum):
-    color1 = '#7a9c38'  # green
-    color2 = '#cb493f'  # red
-    color3 = '#d17109'  # orange
-    color4 = '#4D8CDC'  # blue
-    color5 = '#5f6bfe'  # purple
-    color6 = '#6d7e90'  # greyish
-    color7 = '#ffff00'  # bright yellow
+    color1 = "#7a9c38"  # green
+    color2 = "#cb493f"  # red
+    color3 = "#d17109"  # orange
+    color4 = "#4D8CDC"  # blue
+    color5 = "#5f6bfe"  # purple
+    color6 = "#6d7e90"  # greyish
+    color7 = "#ffff00"  # bright yellow
 
 
-PaleGray = '#d7d6d5'
-DarkGray = '#35322f'
-MediumGray = '#5d5b59'
-DoubleDarkGray = '#1e1b18'
+ButtonHoverIntensity = 110
+
+
+PaleGray = "#d7d6d5"
+DarkGray = "#35322f"
+MediumGray = "#5d5b59"
+DoubleDarkGray = "#1e1b18"
+
+DarkModeMediumGray = "#2b2b2b"
 
 
 ExtensionColorDict = {
@@ -380,10 +409,16 @@ ExtensionColorDict = {
     FileExtension.video: CustomColors.color2,
     FileExtension.jpeg: CustomColors.color4,
     FileExtension.heif: CustomColors.color5,
-    FileExtension.other_photo: CustomColors.color5
+    FileExtension.other_photo: CustomColors.color5,
 }
 
 
+class SyncButtonState(IntEnum):
+    active = 1
+    inactive = 2
+    regular = 3
+
+
 def extensionColor(ext_type: FileExtension) -> QColor:
     try:
         return QColor(ExtensionColorDict[ext_type].value)
@@ -393,7 +428,7 @@ def extensionColor(ext_type: FileExtension) -> QColor:
 
 FileTypeColorDict = {
     FileType.photo: CustomColors.color1,
-    FileType.video: CustomColors.color2
+    FileType.video: CustomColors.color1,  # was color2
 }
 
 
@@ -438,10 +473,12 @@ class StandardFileLocations(Enum):
     downloads = 8
 
 
-
 max_remembered_destinations = 10
 
 ThumbnailBackgroundName = MediumGray
+DarkModeThumbnailBackgroundName = DarkModeMediumGray
+HeaderBackgroundName = ThumbnailBackgroundName
+DarkModeHeaderBackgroundName = DarkModeThumbnailBackgroundName
 EmptyViewHeight = 20
 
 DeviceDisplayPadding = 6
@@ -460,7 +497,7 @@ def minPanelWidth() -> int:
     """
     Minimum width of panels on left and right side of main window.
 
-    Derived from standard font size.
+    Derived from standard font size. Typically around 250 pixels or so.
 
     :return: size in pixels
     """
@@ -488,75 +525,33 @@ def standardProgressBarWidth() -> int:
     return int(QFontMetrics(QFont()).height() * 20)
 
 
-# Be sure to update gvfs_controls_mounts() if updating this
-class Desktop(Enum):
-    gnome = 1
-    unity = 2
-    cinnamon = 3
-    kde = 4
-    xfce = 5
-    mate = 6
-    lxde = 7
-    lxqt = 8
-    ubuntugnome = 9
-    popgnome = 10
-    deepin = 11
-    zorin = 12
-    unknown = 13
-
-
-class FileManagerType(Enum):
-    regular = 1
-    select = 2
-    dir_only_uri = 3
-    show_item = 4
-
-
-FileManagerBehavior = dict(
-    nautilus=FileManagerType.regular,
-    dolphin=FileManagerType.select,
-    caja=FileManagerType.dir_only_uri,
-    thunar=FileManagerType.dir_only_uri,
-    nemo=FileManagerType.regular,
-    pcmanfm=FileManagerType.dir_only_uri,
-)
-FileManagerBehavior['pcmanfm-qt'] = FileManagerType.dir_only_uri
-FileManagerBehavior['dde-file-manager'] = FileManagerType.show_item
-
-
-DefaultFileBrowserFallback = dict(
-    gnome='nautilus',
-    ubuntugnome='nautilus',
-    popgnome='nautilus',
-    unity='nautilus',
-    kde='dolphin',
-    cinnamon='nemo',
-    mate='caja',
-    xfce='thunar',
-    lxde='pcmanfm',
-    lxqt='pcmanfm-qt',
-    deepin='dde-file-manager',
-)
-
-
+# Sync with value in install.py
 class Distro(Enum):
-    debian = 1
-    ubuntu = 2
-    fedora = 3
-    neon = 4
-    linuxmint = 5
-    zorin = 6
-    arch = 7
-    opensuse = 8
-    manjaro = 9
-    galliumos = 10
-    peppermint = 11
-    antergos = 12
-    elementary = 13
-    centos = 14
-    gentoo = 15
-    deepin = 16
-    unknown = 20
+    debian = auto()
+    ubuntu = auto()
+    fedora = auto()
+    neon = auto()
+    linuxmint = auto()
+    zorin = auto()
+    arch = auto()
+    opensuse = auto()
+    manjaro = auto()
+    galliumos = auto()
+    peppermint = auto()
+    elementary = auto()
+    centos_stream9 = auto()
+    centos_stream8 = auto()
+    centos8 = auto()
+    centos7 = auto()
+    gentoo = auto()
+    deepin = auto()
+    kylin = auto()
+    popos = auto()
+    debian_derivative = auto()
+    ubuntu_derivative = auto()
+    fedora_derivative = auto()
+    raspbian = auto()
+    unknown = auto()
 
 
 orientation_offset = dict(
@@ -571,13 +566,13 @@ orientation_offset = dict(
     nrw=94,
     orf=132,
     pef=118,
-    raf=208,
+    raf=8000000,
     raw=742404,
     rw2=1004548,
     sr2=82,
-    srw=46
+    srw=46,
 )
-orientation_offset['3fr'] = 132
+orientation_offset["3fr"] = 132
 
 orientation_offset_exiftool = dict(
     arw=350,
@@ -597,9 +592,9 @@ orientation_offset_exiftool = dict(
     raw=548,
     rw2=709636,
     sr2=276,
-    srw=126
+    srw=126,
 )
-orientation_offset_exiftool['3fr'] = 376
+orientation_offset_exiftool["3fr"] = 376
 
 datetime_offset = dict(
     arw=1540,
@@ -612,7 +607,7 @@ datetime_offset = dict(
     nrw=1540,
     orf=6660,
     pef=836,
-    raf=1796,
+    raf=8000000,
     raw=964,
     rw2=3844,
     sr2=836,
@@ -624,8 +619,8 @@ datetime_offset = dict(
     avi=50000,
     mov=250000,
 )
-datetime_offset['3fr'] = 1540
-datetime_offset['3gp'] = 5000
+datetime_offset["3fr"] = 1540
+datetime_offset["3gp"] = 5000
 
 datetime_offset_exiftool = dict(
     arw=1540,
@@ -652,8 +647,8 @@ datetime_offset_exiftool = dict(
     avi=50000,
     mov=250000,
 )
-datetime_offset_exiftool['3fr'] = 1042
-datetime_offset_exiftool['3gp'] = 5000
+datetime_offset_exiftool["3fr"] = 1042
+datetime_offset_exiftool["3gp"] = 5000
 
 all_tags_offset = dict(
     arw=1848,
@@ -666,13 +661,13 @@ all_tags_offset = dict(
     nrw=1848,
     orf=812242,
     pef=1042,
-    raf=13522,
+    raf=8000000,
     raw=890885,
     rw2=1205458,
     sr2=1080,
     srw=614,
 )
-all_tags_offset['3fr'] = 1848
+all_tags_offset["3fr"] = 1848
 
 all_tags_offset_exiftool = dict(
     arw=1540,
@@ -697,9 +692,9 @@ all_tags_offset_exiftool = dict(
     mt2=1300000,
     m2ts=1300000,
     avi=50000,
-    mov=250000
+    mov=250000,
 )
-all_tags_offset_exiftool['3fr'] = 1042
+all_tags_offset_exiftool["3fr"] = 1042
 
 thumbnail_offset = dict(
     jpg=100000,
@@ -723,7 +718,7 @@ thumbnail_offset_exiftool = dict(
     mrw=84792,
     nef=77213623,
     nrw=45470,
-    raf=84792,
+    raf=8000000,
     raw=890885,
     rw2=1205458,
     sr2=222418,
@@ -740,7 +735,6 @@ thumbnail_offset_exiftool = dict(
 )
 
 
-
 class RememberThisMessage(Enum):
     remember_choice = 1
     do_not_ask_again = 2
@@ -784,13 +778,25 @@ class ScalingAction(Enum):
 
 
 class ScalingDetected(Enum):
-    qt = 1
-    gdk = 2
-    qt_and_gdk = 3
+    Qt = 1
+    Xsetting = 2
+    Qt_and_Xsetting = 3
     undetected = 4
 
 
-# Use the character . to for download_name and path to indicate the user manually marked a
-# file as previously downloaded
-manually_marked_previously_downloaded = '.'
+# Use the character . to for download_name and path to indicate the user manually
+# marked a file as previously downloaded
+manually_marked_previously_downloaded = "."
 
+filtered_file_browser_directories = {
+    "$RECYCLE.BIN",
+    "System Volume Information",
+    "msdownld.tmp",
+}
+
+
+non_system_root_folders = [
+    "/home",
+    "/media",
+    "/mnt",
+]
