@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2011-2020 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2011-2022 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -31,11 +31,11 @@ By default, will set extractors to get the file's metadata time if
 the metadata time is not already found in the rpd_file.
 """
 
-__author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2011-2020, Damon Lynch"
+__author__ = "Damon Lynch"
+__copyright__ = "Copyright 2011-2022, Damon Lynch"
 
 try:
-    using_injected = 'profile' in dict(__builtins__)
+    using_injected = "profile" in dict(__builtins__)
 except:
     using_injected = False
 finally:
@@ -47,6 +47,7 @@ finally:
 
             return inner
 
+
 import os
 import sys
 import logging
@@ -59,39 +60,40 @@ import zmq
 from PyQt5.QtGui import QImage
 from PyQt5.QtCore import QSize
 import psutil
-import gphoto2 as gp
-
-try:
-    import rawkit
-
-    have_rawkit = True
-except ImportError:
-    have_rawkit = False
 
 from raphodo.rpdfile import RPDFile
 from raphodo.interprocess import (
-    WorkerInPublishPullPipeline, GenerateThumbnailsArguments, GenerateThumbnailsResults,
-    ThumbnailExtractorArgument
+    WorkerInPublishPullPipeline,
+    GenerateThumbnailsArguments,
+    GenerateThumbnailsResults,
+    ThumbnailExtractorArgument,
 )
 from raphodo.constants import (
-    FileType, ThumbnailSize, ThumbnailCacheStatus, ThumbnailCacheDiskStatus, ExtractionTask,
-    ExtractionProcessing, orientation_offset, thumbnail_offset, ThumbnailCacheOrigin,
-    datetime_offset, datetime_offset_exiftool, thumbnail_offset_exiftool
-)
-from raphodo.camera import (
-    Camera, CameraProblemEx, gphoto2_python_logging
+    FileType,
+    ThumbnailSize,
+    ThumbnailCacheStatus,
+    ThumbnailCacheDiskStatus,
+    ExtractionTask,
+    ExtractionProcessing,
+    orientation_offset,
+    thumbnail_offset,
+    ThumbnailCacheOrigin,
+    datetime_offset,
+    datetime_offset_exiftool,
+    thumbnail_offset_exiftool,
 )
+from raphodo.camera import Camera, CameraProblemEx, gphoto2_python_logging
 from raphodo.cache import ThumbnailCacheSql, FdoCacheLarge
-from raphodo.utilities import (GenerateRandomFileName, create_temp_dir, CacheDirs)
-from raphodo.preferences import Preferences
+from raphodo.utilities import GenerateRandomFileName, create_temp_dir, CacheDirs
+from raphodo.prefs.preferences import Preferences
 from raphodo.rescan import RescanCamera
-from raphodo.fileformats import use_exiftool_on_photo
+from raphodo.metadata.fileformats import use_exiftool_on_photo
 from raphodo.heif import have_heif_module
 
 
 def cache_dir_name(device_name: str) -> str:
     """Generate a directory name for a temporary file cache"""
-    return 'rpd-cache-{}-'.format(device_name[:10].replace(' ', '_'))
+    return "rpd-cache-{}-".format(device_name[:10].replace(" ", "_"))
 
 
 def split_list(alist: list, wanted_parts=2):
@@ -103,8 +105,10 @@ def split_list(alist: list, wanted_parts=2):
     :return: the split lists
     """
     length = len(alist)
-    return [alist[i * length // wanted_parts: (i + 1) * length // wanted_parts]
-            for i in range(wanted_parts)]
+    return [
+        alist[i * length // wanted_parts : (i + 1) * length // wanted_parts]
+        for i in range(wanted_parts)
+    ]
 
 
 def split_indexes(length: int):
@@ -159,8 +163,7 @@ def get_temporal_gaps_and_sequences(rpd_files, temporal_span):
         gaps = [prev]
         sequences = []
         for i, rpd_file in enumerate(rpd_files[1:]):
-            if rpd_file.modification_time - prev.modification_time > \
-                    temporal_span:
+            if rpd_file.modification_time - prev.modification_time > temporal_span:
                 gaps.append(rpd_file)
             else:
                 sequences.append(rpd_file)
@@ -189,15 +192,17 @@ class GetThumbnailFromCache:
 
     def image_large_enough(self, size: QSize) -> bool:
         """Check if image is equal or bigger than thumbnail size."""
-        return (size.width() >= self.thumbnail_size_needed.width() or
-                size.height() >= self.thumbnail_size_needed.height())
+        return (
+            size.width() >= self.thumbnail_size_needed.width()
+            or size.height() >= self.thumbnail_size_needed.height()
+        )
 
-    def get_from_cache(self, rpd_file: RPDFile,
-                       use_thumbnail_cache: bool = True
-                       ) -> Tuple[ExtractionTask, bytes, str, ThumbnailCacheOrigin]:
+    def get_from_cache(
+        self, rpd_file: RPDFile, use_thumbnail_cache: bool = True
+    ) -> Tuple[ExtractionTask, bytes, str, ThumbnailCacheOrigin]:
         """
-        Attempt to get a thumbnail for the file from the Rapid Photo Downloader thumbnail cache
-        or from the FreeDesktop.org 256x256 thumbnail cache.
+        Attempt to get a thumbnail for the file from the Rapid Photo Downloader
+        thumbnail cache or from the FreeDesktop.org 256x256 thumbnail cache.
 
         :param rpd_file:
         :param use_thumbnail_cache: whether to use the
@@ -206,7 +211,7 @@ class GetThumbnailFromCache:
 
         task = ExtractionTask.undetermined
         thumbnail_bytes = None
-        full_file_name_to_work_on = ''
+        full_file_name_to_work_on = ""
         origin = None  # type: Optional[ThumbnailCacheOrigin]
 
         # Attempt to get thumbnail from Thumbnail Cache
@@ -216,7 +221,8 @@ class GetThumbnailFromCache:
                 full_file_name=rpd_file.full_file_name,
                 mtime=rpd_file.modification_time,
                 size=rpd_file.size,
-                camera_model=rpd_file.camera_model)
+                camera_model=rpd_file.camera_model,
+            )
             rpd_file.thumbnail_cache_status = get_thumbnail.disk_status
             if get_thumbnail.disk_status != ThumbnailCacheDiskStatus.not_found:
                 origin = ThumbnailCacheOrigin.thumbnail_cache
@@ -227,22 +233,25 @@ class GetThumbnailFromCache:
                 elif get_thumbnail.disk_status == ThumbnailCacheDiskStatus.found:
                     rpd_file.thumbnail_cache_status = ThumbnailCacheDiskStatus.found
                     if get_thumbnail.orientation_unknown:
-                        rpd_file.thumbnail_status = ThumbnailCacheStatus.orientation_unknown
+                        rpd_file.thumbnail_status = (
+                            ThumbnailCacheStatus.orientation_unknown
+                        )
                     else:
                         rpd_file.thumbnail_status = ThumbnailCacheStatus.ready
-                    with open(get_thumbnail.path, 'rb') as thumbnail:
+                    with open(get_thumbnail.path, "rb") as thumbnail:
                         thumbnail_bytes = thumbnail.read()
 
         # Attempt to get thumbnail from large FDO Cache if not found in Thumbnail Cache
-        # and it's not being downloaded directly from a camera (if it's from a camera, it's
-        # not going to be in the FDO cache)
+        # and it's not being downloaded directly from a camera (if it's from a camera,
+        # it's not going to be in the FDO cache)
 
         if task == ExtractionTask.undetermined and not rpd_file.from_camera:
             get_thumbnail = self.fdo_cache_large.get_thumbnail(
                 full_file_name=rpd_file.full_file_name,
                 modification_time=rpd_file.modification_time,
                 size=rpd_file.size,
-                camera_model=rpd_file.camera_model)
+                camera_model=rpd_file.camera_model,
+            )
             if get_thumbnail.disk_status == ThumbnailCacheDiskStatus.found:
                 rpd_file.fdo_thumbnail_256_name = get_thumbnail.path
                 thumb = get_thumbnail.thumbnail  # type: QImage
@@ -258,15 +267,12 @@ class GetThumbnailFromCache:
 
 # How much of the file should be read in from local disk and thus cached
 # by they kernel
-cached_read = dict(
-    cr2=260 * 1024,
-    dng=504 * 1024,
-    nef=400 * 1024
-)
+cached_read = dict(cr2=260 * 1024, dng=504 * 1024, nef=400 * 1024)
 
 
-def preprocess_thumbnail_from_disk(rpd_file: RPDFile,
-                                   processing: Set[ExtractionProcessing]) -> ExtractionTask:
+def preprocess_thumbnail_from_disk(
+    rpd_file: RPDFile, processing: Set[ExtractionProcessing]
+) -> ExtractionTask:
     """
     Determine how to get a thumbnail from a photo or video that is not on a camera
     (although it may have directly come from there during the download process)
@@ -288,8 +294,8 @@ def preprocess_thumbnail_from_disk(rpd_file: RPDFile,
                 else:
                     task = ExtractionTask.load_heif_and_exif_directly
                 processing.add(ExtractionProcessing.resize)
-                # For now, do not orient, as it seems pyheif or libheif does that automatically
-                # processing.add(ExtractionProcessing.orient)
+                # For now, do not orient, as it seems pyheif or libheif does that
+                # automatically processing.add(ExtractionProcessing.orient)
             else:
                 # We have no way to convert the file
                 task = ExtractionTask.bypass
@@ -322,12 +328,13 @@ def preprocess_thumbnail_from_disk(rpd_file: RPDFile,
         if bytes_to_read:
             if not rpd_file.download_full_file_name:
                 try:
-                    with open(rpd_file.full_file_name, 'rb') as photo:
+                    with open(rpd_file.full_file_name, "rb") as photo:
                         # Bring the file into the operating system's disk cache
                         photo.read(bytes_to_read)
                 except FileNotFoundError:
                     logging.error(
-                        "The download file %s does not exist", rpd_file.download_full_file_name
+                        "The download file %s does not exist",
+                        rpd_file.download_full_file_name,
                     )
     else:
         # video
@@ -350,10 +357,9 @@ def preprocess_thumbnail_from_disk(rpd_file: RPDFile,
 
 
 class GenerateThumbnails(WorkerInPublishPullPipeline):
-
     def __init__(self) -> None:
         self.random_file_name = GenerateRandomFileName()
-        super().__init__('Thumbnails')
+        super().__init__("Thumbnails")
 
     def cache_full_size_file_from_camera(self, rpd_file: RPDFile) -> bool:
         """
@@ -376,7 +382,7 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                 dest_full_filename=cache_full_file_name,
                 progress_callback=None,
                 check_for_command=self.check_for_controller_directive,
-                return_file_bytes=False
+                return_file_bytes=False,
             )
         except CameraProblemEx as e:
             # TODO report error
@@ -398,7 +404,7 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                 dir_name=rpd_file.path,
                 file_name=rpd_file.name,
                 chunk_size_in_bytes=min(offset, rpd_file.size),
-                dest_full_filename=cache_full_file_name
+                dest_full_filename=cache_full_file_name,
             )
             rpd_file.temp_cache_full_file_chunk = cache_full_file_name
             return True
@@ -406,11 +412,13 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
             # TODO problem reporting
             return False
 
-    def extract_photo_video_from_camera(self,
-                                        rpd_file: RPDFile,
-                                        entire_file_required: bool,
-                                        full_file_name_to_work_on,
-                                        using_exiftool: bool) -> Tuple[ExtractionTask, str, bool]:
+    def extract_photo_video_from_camera(
+        self,
+        rpd_file: RPDFile,
+        entire_file_required: bool,
+        full_file_name_to_work_on,
+        using_exiftool: bool,
+    ) -> Tuple[ExtractionTask, str, bool]:
         """
         Extract part of a photo of video to be able to get the orientation
         and date time metadata, if and only if we know how much of the file
@@ -443,7 +451,9 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                 offset = thumbnail_offset.get(rpd_file.extension)
             if offset:
                 if using_exiftool:
-                    offset = max(offset, datetime_offset_exiftool.get(rpd_file.extension))
+                    offset = max(
+                        offset, datetime_offset_exiftool.get(rpd_file.extension)
+                    )
                 else:
                     offset = max(offset, datetime_offset.get(rpd_file.extension))
 
@@ -470,8 +480,10 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
         except SystemExit as e:
             sys.exit(e)
         except Exception:
-            if hasattr(self, 'device_name'):
-                logging.error("Exception generating thumbnails for %s", self.device_name)
+            if hasattr(self, "device_name"):
+                logging.error(
+                    "Exception generating thumbnails for %s", self.device_name
+                )
             else:
                 logging.error("Exception generating thumbnails")
             logging.exception("Traceback:")
@@ -480,7 +492,9 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
         self.camera = None
         arguments = pickle.loads(self.content)  # type: GenerateThumbnailsArguments
         self.device_name = arguments.name
-        logging.info("Generating %s thumbnails for %s", len(arguments.rpd_files), arguments.name)
+        logging.info(
+            "Generating %s thumbnails for %s", len(arguments.rpd_files), arguments.name
+        )
         if arguments.log_gphoto2:
             self.gphoto2_logging = gphoto2_python_logging()
 
@@ -492,15 +506,18 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
         # Whether we must use ExifTool to read photo metadata
         force_exiftool = self.prefs.force_exiftool
 
-        # If the entire photo or video is required to extract the thumbnail, which is determined
-        # when extracting sample metadata from a photo or video during the device scan
+        # If the entire photo or video is required to extract the thumbnail, which is
+        # determined when extracting sample metadata from a photo or video during the
+        # device scan
         entire_photo_required = arguments.entire_photo_required
         entire_video_required = arguments.entire_video_required
 
         # Access and generate Rapid Photo Downloader thumbnail cache
         use_thumbnail_cache = self.prefs.use_thumbnail_cache
 
-        thumbnail_caches = GetThumbnailFromCache(use_thumbnail_cache=use_thumbnail_cache)
+        thumbnail_caches = GetThumbnailFromCache(
+            use_thumbnail_cache=use_thumbnail_cache
+        )
 
         photo_cache_dir = video_cache_dir = None
         cache_file_from_camera = force_exiftool
@@ -512,7 +529,7 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
 
         # Must sort files by modification time prior to temporal analysis needed to figure out
         # which thumbnails to prioritize
-        rpd_files = sorted(rpd_files, key=attrgetter('modification_time'))
+        rpd_files = sorted(rpd_files, key=attrgetter("modification_time"))
 
         time_span = arguments.proximity_seconds
 
@@ -531,21 +548,25 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
 
         if arguments.camera is not None:
             self.camera = Camera(
-                model=arguments.camera, port=arguments.port,
-                specific_folders=self.prefs.folders_to_scan
+                model=arguments.camera,
+                port=arguments.port,
+                is_mtp_device=arguments.is_mtp_device,
+                specific_folders=self.prefs.folders_to_scan,
             )
 
             if not self.camera.camera_initialized:
                 # There is nothing to do here: exit!
                 logging.debug(
-                    "Prematurely exiting thumbnail generation due to lack of access to camera %s",
-                    arguments.camera
+                    "Prematurely exiting thumbnail generation due to lack of access to "
+                    "camera %s",
+                    arguments.camera,
                 )
                 self.content = pickle.dumps(
                     GenerateThumbnailsResults(
                         scan_id=arguments.scan_id,
                         camera_removed=True,
-                    ), pickle.HIGHEST_PROTOCOL
+                    ),
+                    pickle.HIGHEST_PROTOCOL,
                 )
                 self.send_message_to_sink()
                 self.disconnect_logging()
@@ -554,32 +575,38 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
 
             if not cache_file_from_camera:
                 for rpd_file in rpd_files:
-                    if use_exiftool_on_photo(rpd_file.extension,
-                                             preview_extraction_irrelevant=False):
+                    if use_exiftool_on_photo(
+                        rpd_file.extension, preview_extraction_irrelevant=False
+                    ):
                         cache_file_from_camera = True
                         break
 
-            must_make_cache_dirs = (not self.camera.can_fetch_thumbnails or cache_file_from_camera)
+            must_make_cache_dirs = (
+                not self.camera.can_fetch_thumbnails or cache_file_from_camera
+            )
 
-            if must_make_cache_dirs or arguments.need_video_cache_dir \
-                    or arguments.need_photo_cache_dir:
+            if (
+                must_make_cache_dirs
+                or arguments.need_video_cache_dir
+                or arguments.need_photo_cache_dir
+            ):
                 # If downloading complete copy of the files to
                 # generate previews, then may as well cache them to speed up
                 # the download process
                 self.photo_cache_dir = create_temp_dir(
                     folder=arguments.cache_dirs.photo_cache_dir,
-                    prefix=cache_dir_name(self.device_name)
+                    prefix=cache_dir_name(self.device_name),
                 )
                 self.video_cache_dir = create_temp_dir(
                     folder=arguments.cache_dirs.video_cache_dir,
-                    prefix=cache_dir_name(self.device_name)
+                    prefix=cache_dir_name(self.device_name),
                 )
                 cache_dirs = CacheDirs(self.photo_cache_dir, self.video_cache_dir)
                 self.content = pickle.dumps(
                     GenerateThumbnailsResults(
-                        scan_id=arguments.scan_id,
-                        cache_dirs=cache_dirs
-                    ), pickle.HIGHEST_PROTOCOL
+                        scan_id=arguments.scan_id, cache_dirs=cache_dirs
+                    ),
+                    pickle.HIGHEST_PROTOCOL,
                 )
                 self.send_message_to_sink()
 
@@ -593,12 +620,15 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
             if rescan.missing_rpd_files:
                 logging.error(
                     "%s files could not be relocated on %s",
-                    len(rescan.missing_rpd_files), self.camera.display_name
+                    len(rescan.missing_rpd_files),
+                    self.camera.display_name,
                 )
                 for rpd_file in rescan.missing_rpd_files:  # type: RPDFile
                     self.content = pickle.dumps(
-                        GenerateThumbnailsResults(rpd_file=rpd_file, thumbnail_bytes=None),
-                        pickle.HIGHEST_PROTOCOL
+                        GenerateThumbnailsResults(
+                            rpd_file=rpd_file, thumbnail_bytes=None
+                        ),
+                        pickle.HIGHEST_PROTOCOL,
                     )
                     self.send_message_to_sink()
 
@@ -608,7 +638,7 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
 
             exif_buffer = None
             file_to_work_on_is_temporary = False
-            secondary_full_file_name = ''
+            secondary_full_file_name = ""
             processing = set()  # type: Set[ExtractionProcessing]
 
             # Attempt to get thumbnail from Thumbnail Cache
@@ -622,7 +652,8 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                 else:
                     assert origin == ThumbnailCacheOrigin.fdo_cache
                     logging.debug(
-                        "Thumbnail for %s found in large FDO cache", rpd_file.full_file_name
+                        "Thumbnail for %s found in large FDO cache",
+                        rpd_file.full_file_name,
                     )
                     from_fdo_cache += 1
                     processing.add(ExtractionProcessing.resize)
@@ -637,7 +668,8 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                         secondary_full_file_name = rpd_file.full_file_name
                         logging.debug(
                             "Although thumbnail found in the cache, tasked to extract "
-                            "file time recorded in metadata from %s", secondary_full_file_name
+                            "file time recorded in metadata from %s",
+                            secondary_full_file_name,
                         )
             if task == ExtractionTask.undetermined:
                 # Thumbnail was not found in any cache: extract it
@@ -651,7 +683,9 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                             if self.cache_full_size_file_from_camera(rpd_file):
                                 task = ExtractionTask.load_heif_and_exif_directly
                                 processing.add(ExtractionProcessing.resize)
-                                full_file_name_to_work_on = rpd_file.cache_full_file_name
+                                full_file_name_to_work_on = (
+                                    rpd_file.cache_full_file_name
+                                )
                                 # For now, do not orient, as it seems pyheif or libheif does
                                 # that automatically.
                                 # processing.add(ExtractionProcessing.orient)
@@ -669,32 +703,43 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                                     thumbnail_bytes = None
                             else:
 
-                                if force_exiftool or use_exiftool_on_photo(rpd_file.extension,
-                                                           preview_extraction_irrelevant=False):
-                                    task, full_file_name_to_work_on, \
-                                        file_to_work_on_is_temporary =\
-                                        self.extract_photo_video_from_camera(
-                                            rpd_file, entire_photo_required,
-                                            full_file_name_to_work_on,
-                                            True
+                                if force_exiftool or use_exiftool_on_photo(
+                                    rpd_file.extension,
+                                    preview_extraction_irrelevant=False,
+                                ):
+                                    (
+                                        task,
+                                        full_file_name_to_work_on,
+                                        file_to_work_on_is_temporary,
+                                    ) = self.extract_photo_video_from_camera(
+                                        rpd_file,
+                                        entire_photo_required,
+                                        full_file_name_to_work_on,
+                                        True,
+                                    )
+                                    if (
+                                        task
+                                        == ExtractionTask.load_from_bytes_metadata_from_temp_extract
+                                    ):
+                                        secondary_full_file_name = (
+                                            full_file_name_to_work_on
                                         )
-                                    if task == ExtractionTask.load_from_bytes_metadata_from_temp_extract:
-                                        secondary_full_file_name = full_file_name_to_work_on
                                         file_to_work_on_is_temporary = False
 
                                 else:
-                                    # gPhoto2 does not know how to get RAW thumbnails, so we do that
-                                    # part ourselves
-                                    if rpd_file.extension == 'crw':
-                                        # Could cache this file, since reading its entirety
-                                        # But does anyone download a CRW file from the camera these
-                                        # days?!
+                                    # gPhoto2 does not know how to get RAW thumbnails,
+                                    # so we do that part ourselves
+                                    if rpd_file.extension == "crw":
+                                        # Could cache this file, since reading its
+                                        # entirety But does anyone download a CRW file
+                                        # from the camera these days?!
                                         bytes_to_read = rpd_file.size
                                     else:
                                         bytes_to_read = min(
-                                            rpd_file.size, orientation_offset.get(
+                                            rpd_file.size,
+                                            orientation_offset.get(
                                                 rpd_file.extension, 500
-                                            )
+                                            ),
                                         )
                                     exif_buffer = self.camera.get_exif_extract(
                                         rpd_file.path, rpd_file.name, bytes_to_read
@@ -712,13 +757,11 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                             # Many (all?) jpegs from phones don't include jpeg previews,
                             # so need to render from the entire jpeg itself. Slow!
 
-                            # If rawkit is not installed, then extract merely a part of
-                            # phone's raw format, and try to extract the jpeg preview
-                            # from it (which probably doesn't exist!). This is fast.
-                            # If have rawkit, download and render an image from the
-                            # RAW
+                            # For raw, extract merely a part of phone's raw format, and
+                            # try to extract the jpeg preview from it (which probably
+                            # doesn't exist!). This is fast.
 
-                            if not rpd_file.is_jpeg() and not have_rawkit:
+                            if not rpd_file.is_jpeg():
                                 bytes_to_read = thumbnail_offset.get(rpd_file.extension)
                                 if bytes_to_read:
                                     exif_buffer = self.camera.get_exif_extract(
@@ -726,8 +769,10 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                                     )
                                     task = ExtractionTask.load_from_exif_buffer
                                     processing.add(ExtractionProcessing.orient)
-                            if (task == ExtractionTask.undetermined and
-                                    self.cache_full_size_file_from_camera(rpd_file)):
+                            if (
+                                task == ExtractionTask.undetermined
+                                and self.cache_full_size_file_from_camera(rpd_file)
+                            ):
                                 if rpd_file.is_jpeg():
                                     task = ExtractionTask.load_file_and_exif_directly
                                     processing.add(ExtractionProcessing.resize)
@@ -736,7 +781,9 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                                     task = ExtractionTask.load_from_exif
                                     processing.add(ExtractionProcessing.resize)
                                     processing.add(ExtractionProcessing.orient)
-                                full_file_name_to_work_on = rpd_file.cache_full_file_name
+                                full_file_name_to_work_on = (
+                                    rpd_file.cache_full_file_name
+                                )
                             else:
                                 # Failed to generate thumbnail
                                 task = ExtractionTask.bypass
@@ -750,54 +797,73 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                                 offset = rpd_file.size
                             else:
                                 offset = datetime_offset.get(rpd_file.extension)
-                                # If there is no offset, there is no point trying to extract the
-                                # metadata time from part of the video. It's not ideal,
-                                # but if this is from a camera on which there were any other files
-                                # we can assume we've got a somewhat accurate date time for it from
-                                # the modification time.
-                                # The only exception is if the video file is not that big, in which
-                                # case it's worth reading in its entirety:
+                                # If there is no offset, there is no point trying to
+                                # extract the metadata time from part of the video. It's
+                                # not ideal, but if this is from a camera on which there
+                                # were any other files we can assume we've got a
+                                # somewhat accurate date time for it from the
+                                # modification time. The only exception is if the video
+                                # file is not that big, in which case it's worth reading
+                                # in its entirety:
                                 if offset is None and rpd_file.size < 4000000:
                                     offset = rpd_file.size
 
                             if rpd_file.mdatatime or not offset:
                                 task = ExtractionTask.load_from_bytes
                             elif self.cache_file_chunk_from_camera(rpd_file, offset):
-                                task = ExtractionTask.load_from_bytes_metadata_from_temp_extract
-                                secondary_full_file_name = rpd_file.temp_cache_full_file_chunk
+                                task = (
+                                    ExtractionTask.load_from_bytes_metadata_from_temp_extract
+                                )
+                                secondary_full_file_name = (
+                                    rpd_file.temp_cache_full_file_chunk
+                                )
                             else:
-                                # For some reason was unable to download part of the video file
+                                # For some reason was unable to download part of the
+                                # video file
                                 task = ExtractionTask.load_from_bytes
 
                             try:
-                                thumbnail_bytes = self.camera.get_THM_file(rpd_file.thm_full_name)
+                                thumbnail_bytes = self.camera.get_THM_file(
+                                    rpd_file.thm_full_name
+                                )
                             except CameraProblemEx as e:
                                 # TODO report error
                                 thumbnail_bytes = None
                             processing.add(ExtractionProcessing.strip_bars_video)
                             processing.add(ExtractionProcessing.add_film_strip)
                         else:
-                            task, full_file_name_to_work_on, file_to_work_on_is_temporary = \
-                                self.extract_photo_video_from_camera(
-                                    rpd_file, entire_video_required,
-                                    full_file_name_to_work_on,
-                                    False
-                                )
+                            (
+                                task,
+                                full_file_name_to_work_on,
+                                file_to_work_on_is_temporary,
+                            ) = self.extract_photo_video_from_camera(
+                                rpd_file,
+                                entire_video_required,
+                                full_file_name_to_work_on,
+                                False,
+                            )
                 else:
                     # File is not on a camera
-                    task = preprocess_thumbnail_from_disk(rpd_file=rpd_file, processing=processing)
+                    task = preprocess_thumbnail_from_disk(
+                        rpd_file=rpd_file, processing=processing
+                    )
                     if task != ExtractionTask.bypass:
                         if rpd_file.thm_full_name is not None:
                             full_file_name_to_work_on = rpd_file.thm_full_name
-                            if task == ExtractionTask.load_file_directly_metadata_from_secondary:
+                            if (
+                                task
+                                == ExtractionTask.load_file_directly_metadata_from_secondary
+                            ):
                                 secondary_full_file_name = rpd_file.full_file_name
                         else:
                             full_file_name_to_work_on = rpd_file.full_file_name
 
             if task == ExtractionTask.bypass:
                 self.content = pickle.dumps(
-                    GenerateThumbnailsResults(rpd_file=rpd_file, thumbnail_bytes=thumbnail_bytes),
-                    pickle.HIGHEST_PROTOCOL
+                    GenerateThumbnailsResults(
+                        rpd_file=rpd_file, thumbnail_bytes=thumbnail_bytes
+                    ),
+                    pickle.HIGHEST_PROTOCOL,
                 )
                 self.send_message_to_sink()
 
@@ -818,10 +884,11 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                         file_to_work_on_is_temporary=file_to_work_on_is_temporary,
                         write_fdo_thumbnail=False,
                         send_thumb_to_main=True,
-                        force_exiftool=force_exiftool
+                        force_exiftool=force_exiftool,
                     ),
-                    pickle.HIGHEST_PROTOCOL)
-                self.frontend.send_multipart([b'data', self.content])
+                    pickle.HIGHEST_PROTOCOL,
+                )
+                self.frontend.send_multipart([b"data", self.content])
 
         if arguments.camera:
             self.camera.free_camera()
@@ -833,7 +900,9 @@ class GenerateThumbnails(WorkerInPublishPullPipeline):
                 if not os.listdir(self.video_cache_dir):
                     os.rmdir(self.video_cache_dir)
 
-        logging.debug("Finished phase 1 of thumbnail generation for %s", self.device_name)
+        logging.debug(
+            "Finished phase 1 of thumbnail generation for %s", self.device_name
+        )
         if from_thumb_cache:
             logging.info(
                 "{} of {} thumbnails for {} came from thumbnail cache".format(
